% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
% Dr. Jos Ernesto Rayas-Snchez                                 April 10, 2019
% Research Group on Computer-Aided Engineering of Circuits and Systems (CAECAS)
% Department of Electronics, Systems and Informatics                      ITESO
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
%  Ideal One-Section Impedance Transformer Simulated in Matlab (coarse model)
%
% This function calculates the magnitude and phase of S11 of a one-section 
% impedance transformer with an ideal transmission line. A reference 
% impedance of Zo ohms is used. 
%
% Usage: [psi,R] = ZtranTL_1sec_c_Matlab(P)
%        P(1) = Zo; Reference impedance (ohms).
%        P(2) = ZL; Load impedance (ohms).
%        P(3) = Z1; TL characteristic impedance (ohms).
%        P(4) = epsr; Effective dielectric constant of the TL.
%        P(5) = f0; Reference frequency for wavelength on TL (GHz).
%        P(6) = L1; Electrical length of the TL (degrees).
%        P(7) = IF; Initial frequency (GHz).
%        P(8) = FF; Final frequency (GHz).
%        P(9) = FP; Number of frequency samples.
%        psi: column vector of simulated frequency values (GHz).                    
%        R: matrix containing the circuit response values (S-parameters). 

function [psi,R] = ZtranTL_1sec_c_Matlab(P)

% Reading Parameters
Zo = P(1);   % Reference impedance (ohms).
ZL = P(2);   % Load impedance (ohms).
Z1 = P(3);   % TL characteristic impedance (ohms).
epsr = P(4); % Effective dielectric constant of the TL.
f0 = P(5);   % Reference frequency for wavelength on TL (GHz).
L1 = P(6);   % Electrical length of the TL impedance transformer (degrees).
IF = P(7);   % Initial frequency (GHz).
FF = P(8);   % Final frequency (GHz).
FP = P(9);   % Number of frequency samples.

% Physical Constants
eps0 = 8.854e-12;      % Permittivity of free-space (F/m).
miu0 = 4*pi*1e-7;      % Permeability of free-space (H/m).
c = 1/sqrt(eps0*miu0); % Speed of light in free-space (m/s).

% Preliminary Calculations
vp = c/sqrt(epsr);     % Phase velocity on the on TL (m/s). 
lambda0 = vp/(f0*1e9); % Wavelength at reference frequency on TL (m).
beta0 = 2*pi/lambda0;  % Propagation constant at reference frequency (1/m).
L1r = L1*(pi/180);     % Converting electrical length from degrees to radians.
L1p = L1r/beta0;       % Physical length of the TL transformer (radians).

% Calculating Responses
j = sqrt(-1);
f = linspace(IF,FF,FP); % Row vector of frequencies (GHz).
lambda = vp./(f*1e9);   % Wavelength at each frequency (m).
beta = 2*pi./lambda;    % Propagation constant at each frequency (1/m).
betaLT = beta*L1p;      % TL electrical length at each frequency (rad).
tanbl = tan(betaLT);
Zin = Z1*(ZL + j*Z1*tanbl)./(Z1 + j*ZL*tanbl);
S11cx = (Zin-Zo)./(Zin+Zo);
mS11 = abs(S11cx);
pS11 = (180/pi)*angle(S11cx);

psi = f'; 
R(:,1) = mS11';
R(:,2) = pS11';
